
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "../MachineInterface.h"
#include "../auxbus/auxbus.h"

#define MAX_TRACKS		5

CMachineParameter const paraMode = 
{ 
	pt_byte,										// type
	"Mode",									// short description
	"Modes: 0 Analyse, 1 Ana + Prot, 2 Ana + ASR, 3 Ana + AuxbusASR, 4 ASR, 5 AuxbusASR, 6 Prot",		// description
	0,												// MinValue	
	6,											// MaxValue
	0xff,											// NoValue
	MPF_STATE,										// Flags
	1											// Default value
};

CMachineParameter const paraThreshold = 
{ 
	pt_word,										// type
	"Threshold",									// short description
	"Threshold: 0db=0x8000(32768), +14db=0xFFFE(65534)",			// description
	0,												// MinValue	
	65534,											// MaxValue
	65535,											// NoValue
	MPF_STATE,										// Flags
	0x8000											// Default value
};

CMachineParameter const paraDelta = 
{ 
	pt_word,										// type
	"Max Delta",									// short description
	"Max difference: 100%=0xA000(40960), 249%=0xFFFE(65534)",						// description
	0,												// MinValue	
	65534,											// MaxValue
	65535,											// NoValue
	MPF_STATE,										// Flags
	0xa000											// Default value
};

CMachineParameter const paraSustainLevel = 
{ 
	pt_word,										// type
	"Level",									// short description
	"Sustain level: 0dB=0x8000(32768), +19.1dB=0xFFFE(65534)",						// description
	0,												// MinValue	
	65534,											// MaxValue
	65535,											// NoValue
	MPF_STATE,										// Flags
	0												// Default value
};

CMachineParameter const paraAttack = 
{ 
	pt_word,										// type
	"Attack",									// short description
	"Attack time in units",						// description
	0,												// MinValue	
	65534,											// MaxValue
	65535,											// NoValue
	MPF_STATE,										// Flags
	1											// Default value
};

CMachineParameter const paraSustain = 
{ 
	pt_word,									// type
	"Sustain",										// short description
	"Sustain time in units",// description
	0,											// MinValue	
	65534,										// MaxValue
	65535,										// NoValue
	MPF_STATE,									// Flags
	8											// Default value
};


CMachineParameter const paraRelease = 
{ 
	pt_word,									// type
	"Release",									// short description
	"Release time in units",	// description
	0,											// MinValue	
	65534,										// MaxValue
	65535,										// NoValue
	MPF_STATE,									// Flags
	1											// Default value
};


CMachineParameter const paraAttackUnit = 
{ 
	pt_byte,										// type
	"Attack Unit",									// short description
	"Attack Unit: 0=tick (default), 1 ticks/256, 2 samples, 3=ms, 4=seconds",		// description
	0,												// MinValue	
	4,											// MaxValue
	0xff,											// NoValue
	MPF_STATE,										// Flags
	0											// Default value
};

CMachineParameter const paraSustainUnit = 
{ 
	pt_byte,										// type
	"Sustain Unit",									// short description
	"Sustain Unit:0=tick (default), 1 ticks/256, 2 samples, 3=ms, 4=seconds",		// description
	0,												// MinValue	
	4,											// MaxValue
	0xff,											// NoValue
	MPF_STATE,										// Flags
	0											// Default value
};

CMachineParameter const paraReleaseUnit = 
{ 
	pt_byte,										// type
	"Release Unit",									// short description
	"Release Unit: 0=tick (default), 1 ticks/256, 2 samples, 3=ms, 4=seconds",		// description
	0,												// MinValue	
	4,											// MaxValue
	0xff,											// NoValue
	MPF_STATE,										// Flags
	0											// Default value
};


// List of all parameters, track parameters last

CMachineParameter const *pParameters[] = 
{ 
	&paraMode,&paraThreshold, &paraDelta, &paraSustainLevel, &paraAttack, &paraSustain, &paraRelease, &paraAttackUnit, &paraSustainUnit, &paraReleaseUnit
};

/*
// List of all attributes

CMachineAttribute const *pAttributes[] = 
{
	NULL
};
*/

#pragma pack(1)

class gvals
{
public:
	byte mode;
};

class tvals
{
public:
	word threshold;
	word delta;
	word sustainlevel;
	word attack;
	word sustain;
	word release;
	byte attackunit;
	byte sustainunit;
	byte releaseunit;
};

/*
class avals
{
public:

};
*/


#pragma pack()

// Machine's info

CMachineInfo const MacInfo = 
{
	MT_EFFECT,								// type
	MI_VERSION,
	0,										// flags
	1,										// min tracks
	MAX_TRACKS,								// max tracks
	1,										// numGlobalParameters
	9,										// numTrackParameters
	pParameters,							// pointer to list of parameters
	0,										// numAttributes
/*	pAttributes*/ NULL,									// pointer to attributes

#ifdef _DEBUG
	"cheapo protection Debug",			// name for debug
#else
	"cheapo protection",					// name for release build
#endif
	"ch.prot",								// short name
	"Mikko Apo",						// author
	"Show analysis\nReset analysis\nSet AuxChannel\nAbout"
};

class CTrack
{
public:
	byte valAttackUnit,valSustainUnit,valReleaseUnit;
	word valAttack,valSustain,valRelease;
	float valThreshold,valDelta,valSustainLevel;

	unsigned long counter,length;
	double amp_mul_increment,amp_mul;
	float Prev_data,max_burst;

	unsigned long lenAttack,lenSustain,lenRelease;

	int asrMode;

	inline void WorkTrack(float *psample);
	inline void miniWorkTrack(float *psample);
	inline void auxWorkTrack(float *sample,float *auxsample);
private:
};


class mi : public CMachineInterface
{
public:
	mi();

	virtual ~mi();
	virtual void Init(CMachineDataInput * const pi);
	virtual void Tick();
	virtual bool Work(float *psamples, int numsamples, int const mode);

	virtual void SetNumTracks(int const n);

	virtual void Command(int const i);
	/*	virtual void Stop();*/

	virtual char const *DescribeValue(int const param, int const value);

	// auxbus

	void DisconnectAux();
	virtual void Save(CMachineDataOutput * const po);

	// auxbus end
private:
	inline void TickTrack(CTrack *pt, tvals *ptval);
	inline void InitTrack(CTrack *pt);
	unsigned long calculate_length(byte type,word len);

	tvals tval[MAX_TRACKS];

	gvals gval;

private:
	int numTracks;
	CTrack Tracks[MAX_TRACKS];

	int Channel;

//	inline void analysein(float psample);
//	inline void analyseout(float psample);

	float In_maxlevelfound,In_minlevelfound;
	float Out_maxlevelfound,Out_minlevelfound;
	double In_dcsum,Out_dcsum;
	unsigned long dcnum;
	byte running_mode;

	unsigned long milenAttack;
	float mivalSustainLevel;
};

/*
	States:
	asrmode_dry
	asrmode_attack
	asrmode_hold
	asrmode_release

  */

enum {asrmode_Attack_enum, asrmode_Sustain_enum,asrmode_Release_enum};

#define asrmode_Dry (0)
#define asrmode_Attack (1<<asrmode_Attack_enum)
#define asrmode_Sustain (1<<asrmode_Sustain_enum)
#define asrmode_Release (1<<asrmode_Release_enum)

DLL_EXPORTS

mi::mi()
{
	GlobalVals = &gval;
	TrackVals = &tval;
	AttrVals = NULL;

}

// auxbus

mi::~mi()
{
	AB_Disconnect(this);
}

void cb(void *user)
{
	mi *pmi = (mi *)user;
	pmi->DisconnectAux();
}

#define VERSION		1

// auxbus end



// Produces output for analysis

void print_ana2(char *txt, char *txt2, float num)
{
	sprintf(txt,"%s   %s sample: %+.1f (%.1fdB)",txt,txt2,num,num?(float)20.0*log10(fabs(num)/( (num>0)?32767.0:32768.0) ):0.0);
	if(num>32767.0||num<-32768.0)
	{
		sprintf(txt,"%s *** Possible clipping ***",txt);
	}
	sprintf(txt,"%s\n",txt);
}

void print_ana1(char *txt, float max,float min)
{
	print_ana2(txt,"Min",min);
	print_ana2(txt,"Max",max);
}

void print_ana3(char *txt,char *txt2, float max,float min)
{
	sprintf(txt,"%s\"%s\":",txt,txt2);
	if(max>32767.0||min<-32768.0)
	{
	    max=(float)fabs(max);
	    min=(float)fabs(min);
		sprintf(txt,"%s\n     Machine: under %d",txt,(int)(16384*( (max > min) ? (32767.0/max) : (32768.0/min) ) ) );
		sprintf(txt,"%s\n        Master: over ",txt);
		sprintf(txt,"%s %d\n",txt,(int)(log10((max>min)?(32767.0/max):(32768.0/min))*((20.0*16384.0)/(-80.0))));
	} else
	{
		sprintf(txt,"%s\n     Clipping has not been detected.\n",txt);
	}
}

void mi::Command(int const i)
{
	unsigned long secs;
	static char txt[400];
	switch(i)
	{
	case 0:
		if(dcnum)
		{
		  sprintf(txt,"Sound analysis:\n\n");

		  sprintf(txt,"%sStatistical data collected %ld samples ",txt,dcnum);
		  secs=(unsigned int)(dcnum/pMasterInfo->SamplesPerSec);
		  sprintf(txt,"%s (%d:%02d:%02d.%03d)\n\n",txt,secs/3600,(secs%3600)/60,secs%60,(1000*(dcnum%(pMasterInfo->SamplesPerSec)))/pMasterInfo->SamplesPerSec);

		  sprintf(txt,"%sDetected amplitude levels:\nIn:\n",txt);
		  print_ana1(txt,In_maxlevelfound,In_minlevelfound);
		  sprintf(txt,"%s   DC Offset: %.1f (%.1fdB)\n\n",txt,In_dcsum/dcnum,In_dcsum?20.0*log10(fabs(In_dcsum/(dcnum*32768.0))):0.0);
		  if(running_mode)
		  {
		    sprintf(txt,"%sOut:\n",txt);
		    print_ana1(txt,Out_maxlevelfound,Out_minlevelfound);
			sprintf(txt,"%s   DC Offset: %.1f (%.1fdB)\n\n",txt,Out_dcsum/dcnum,Out_dcsum?20.0*log10(fabs(Out_dcsum/(dcnum*32768.0))):0.0);
		  }
		
		  sprintf(txt,"%sSuggested volume slider settings based on\n",txt);
		  print_ana3(txt,"Max In",In_maxlevelfound,In_minlevelfound);
		  if(running_mode)
		  {
		    print_ana3(txt,"Max Out",Out_maxlevelfound,Out_minlevelfound);
		  }
		} else
		{
			sprintf(txt,"No sound analysed.\n");
		}

		pCB->MessageBox(txt);
		break;
	case 1:
		In_maxlevelfound=0;
		In_minlevelfound=0;
		Out_maxlevelfound=0;
		Out_minlevelfound=0;
		Out_dcsum=0;
		In_dcsum=0;
		dcnum=0;
		break;
		// auxbus
	case 2:
		AB_ShowEditor(NULL, &Channel, MacInfo.ShortName, cb, this);
		break;
		// auxbus end
	case 3:
		pCB->MessageBox("cheapo Protector\n\nBuild date: "__DATE__"\nCoded by: Mikko Apo (apo@iki.fi)\nThanks to Oskari and other #buzz developers.\n\nCheck out http://www.iki.fi/apo/buzz/\nfor more buzz stuff.\n\nExcellent skin made by Hymax.");
		break;
	}
}

char const *mi::DescribeValue(int const param, int const value)
{
	int tmp;
	static char txt[16];

	switch(param)
	{
	case 0:
		switch(value)
		{
		case 0: return("Analyse (0)");
		case 1: return("Ana + Prot (1)");
		case 2:	return("Ana + ASR (2)");
		case 3:	return("Ana + AUX(3)");
		case 4: return("AUXASR (4)");
		case 5:	return("ASR (5)");
		case 6: return("Prot (6)");
		}
		break;
	case 1:		// threshold
		if(!value)
		{
			return("-inf dB");
		}
		tmp = ((value>0x8000)? 4*value-3*0x8000 : value);
		sprintf(txt, "%.1fdB", 20*log10(((float)tmp)/(32768.0)));
		break;
	case 2:		// delta
		tmp = ((value>0x8000)? 4*value-3*0x8000 : value);
		sprintf(txt, "%d=%d%%", tmp, (100*tmp)/65534);
		break;
	case 3:		// sustainlevel
		if(!value)
		{
			return("-inf dB");
		}
		tmp = ((value>0x8000)? 8*value-7*0x8000 : value);
		sprintf(txt, "%.1fdB", 20*log10(((float)tmp)/(32768.0)));
		break;
	case 7:
	case 8:
	case 9:
		switch(value)
		{
		case 0: return("ticks");
		case 1: return("ticks/256");
		case 2:	return("samples");
		case 3:	return("ms");
		case 4: return("secs");
		}
	case 4:
	case 5: 
	case 6:
		sprintf(txt,"%ld units",value);
		break;
	}
	return txt;
}

void mi::SetNumTracks(int const n)
{
	if(n>numTracks)
	{
		for (int c = numTracks; c < n; c++)
		{
			InitTrack(Tracks+c);
		}
	}

	numTracks = n;
}


void mi::InitTrack(CTrack *pt)
{
	pt->valThreshold=(float)4.0*paraThreshold.DefValue;
	pt->valDelta=(float)4.0*paraDelta.DefValue;
	pt->valAttack=paraAttack.DefValue;
	pt->valSustain=paraSustain.DefValue;
	pt->valSustainLevel=(float) 4.0*paraSustainLevel.DefValue;
	pt->valRelease=paraRelease.DefValue;
	pt->valAttackUnit=paraAttackUnit.DefValue;
	pt->valSustainUnit=paraSustainUnit.DefValue;
	pt->valReleaseUnit=paraReleaseUnit.DefValue;

	pt->lenAttack=calculate_length(pt->valAttackUnit, pt->valAttack);
	pt->lenSustain=calculate_length(pt->valSustainUnit, pt->valSustain);
	pt->lenRelease=calculate_length(pt->valReleaseUnit, pt->valRelease);

	// other variables
	pt->asrMode=asrmode_Dry;
	pt->Prev_data=0;
	pt->max_burst=0;
	pt->amp_mul=1;
	pt->amp_mul_increment=0;
}

void mi::Init(CMachineDataInput * const pi)
{

	// auxbus
	if (pi != NULL)
	{
		byte ver;
		pi->Read(ver);
		if (ver == VERSION)
		{
			pi->Read(Channel);
			
			if (Channel != -1)
				AB_ConnectOutput(Channel, MacInfo.ShortName, cb, this);
		}
	
	}
	else
	{
		Channel = -1;
	}

	// auxbus end

	numTracks=0;
	Out_maxlevelfound=In_maxlevelfound=-9999999.0;
	Out_minlevelfound=In_minlevelfound=9999999.0;
	In_dcsum=0;
	Out_dcsum=0;
	dcnum=0;
	running_mode=paraMode.DefValue;
}

// auxbus

void mi::Save(CMachineDataOutput * const po)
{
	po->Write((byte)VERSION);
	po->Write(Channel);
}

void mi::DisconnectAux()
{
	MACHINE_LOCK;

	Channel = -1;
}

// auxbus end



// Boy this is ugly

inline void mi::TickTrack(CTrack *pt, tvals *ptval)
{
	if (ptval->threshold != paraThreshold.NoValue)
	{
		pt->valThreshold = (float)( (ptval->threshold>0x8000) ? 4*ptval->threshold-3*0x8000 : ptval->threshold ) ;
	}
	if (ptval->delta != paraDelta.NoValue)
	{
		pt->valDelta = (float)( (ptval->delta>0x8000) ? 4*ptval->delta - 3*0x8000 : ptval->delta);
	}
	if (ptval->sustainlevel != paraSustainLevel.NoValue)
	{
	  pt->valSustainLevel = (float)( (ptval->sustainlevel>0x8000) ? 8*ptval->sustainlevel - 7*0x8000 : ptval->sustainlevel);
	}
	if (ptval->attack != paraAttack.NoValue)
	{
	  pt->valAttack = ptval->attack;
	  pt->lenAttack=calculate_length(pt->valAttackUnit, pt->valAttack);
	}
	if (ptval->sustain != paraSustain.NoValue)
	{
	  pt->valSustain = ptval->sustain;
	  pt->lenSustain=calculate_length(pt->valSustainUnit, pt->valSustain);
	}
	if (ptval->release != paraRelease.NoValue)
	{
	  pt->valRelease = ptval->release;
	  pt->lenRelease=calculate_length(pt->valReleaseUnit, pt->valRelease);
	}
	if (ptval->attackunit != paraAttackUnit.NoValue)
	{
	  pt->valAttackUnit = ptval->attackunit;
	  pt->lenAttack=calculate_length(pt->valAttackUnit, pt->valAttack);
	}
	if (ptval->sustainunit != paraSustainUnit.NoValue)
	{
	  pt->valSustainUnit = ptval->sustainunit;
	  pt->lenSustain=calculate_length(pt->valSustainUnit, pt->valSustain);
	}
	if (ptval->releaseunit != paraReleaseUnit.NoValue)
	{
	  pt->valReleaseUnit = ptval->releaseunit;
	  pt->lenRelease=calculate_length(pt->valReleaseUnit, pt->valRelease);
	}
	switch(pt->asrMode)
	{
	case asrmode_Attack:
		pt->length=pt->lenAttack;
		break;
	case asrmode_Sustain:
		pt->length=pt->lenSustain;
		break;
	case asrmode_Release:
		pt->length=pt->lenRelease;
		break;
	}
}

void mi::Tick()
{
	if (gval.mode != paraMode.NoValue)
	{
      if(running_mode==1||running_mode==6)
	  {
		Tracks[0].asrMode=asrmode_Dry;
		Tracks[0].Prev_data=0;
		Tracks[0].max_burst=0;
		Tracks[0].amp_mul=1;
		Tracks[0].amp_mul_increment=0;
	  }
	  running_mode = gval.mode;
	}
	for (int c = 0; c < numTracks; c++)
	{
		TickTrack(Tracks+c,tval+c);
	}
}

unsigned long mi::calculate_length(byte type, word len)
{
	unsigned long length;
	switch(type)
	{
	case 0:
		length=len*pMasterInfo->SamplesPerTick;
		break;
	case 1:
		length=(len*pMasterInfo->SamplesPerTick)/256;
		break;
	case 2:
		length=len;
		break;
	case 3:
		length=(len*pMasterInfo->SamplesPerSec)/1000;
		break;
	case 4:
		length=len*pMasterInfo->SamplesPerSec;
		break;
	}
	return length;
}

inline void CTrack::WorkTrack(float *sample)
{
	  //
	  // burst detection
	  //

	  float tmp=(float)fabs(*sample);

	  // if tmp goes ever the threshold or the rise is bigger then valDelta then we need to trigger attack
	  if(max_burst<tmp&&(tmp>valThreshold || fabs(*sample - Prev_data) > valDelta))
	  {
		// if we are on attack then counter stays the same, if we are on different stage of asr
		// then we need to trigger full lenght attack
		if(asrMode!=asrmode_Attack)
			counter=0;
		// max_burst has the biggest value of signal during this attack
		asrMode=asrmode_Attack;	// we need to attack
		max_burst=tmp;
		length=lenAttack;
		if(length&&length!=counter)
		{
			// calculate the needed increment to fade during length
			amp_mul_increment=(valSustainLevel / max_burst - amp_mul)/(length-counter);
		}
	  }

	  // prev_data is used to get the rate of change for the signal
  	  Prev_data=*sample;


	  //
	  // envelope, attack part is in the burst detection
	  //

mode_loop:
	  if(asrMode)
	  {
	    if(++counter>length)
		{
			counter=0;
			// some mode has ended, we need to find to which mode we're going to change
			switch(asrMode)
			{
			  case asrmode_Attack:
				  // here we set the values for sustain
				asrMode=asrmode_Sustain;
				length=lenSustain;
				amp_mul=(double)valSustainLevel/max_burst;
				amp_mul_increment=0;
			  break;
			  case asrmode_Sustain:
				  // here's settings for release
				max_burst=0;
				asrMode=asrmode_Release;
				length=lenRelease;
				if(length)
				{
				  amp_mul_increment=(1-amp_mul)/length;
				}
			  break;
			  case asrmode_Release:
				  // here's normal action
				amp_mul_increment=0;
				amp_mul=1;
				asrMode=asrmode_Dry;
			  break;
			}
			goto mode_loop; // this checks if the length of some parameter is 0
		}

	    amp_mul+=amp_mul_increment;
	  }

	  //
	  // apply envelope
	  //

	  (*sample)*=(float)amp_mul;
}

inline void CTrack::auxWorkTrack(float *sample,float *auxsample)
{
	  //
	  // burst detection
	  //

	  float tmp=(float)fabs(*auxsample);

	  // if tmp goes ever the threshold or the rise is bigger then valDelta then we need to trigger attack
	  // max_burst has the biggest value of signal during this envelope
	  if(max_burst<tmp && (tmp>valThreshold || fabs(*auxsample - Prev_data) > valDelta))
	  {
		// if we are on attack then counter stays the same, if we are on different stage of asr
		// then we need to trigger full lenght attack
		if(asrMode!=asrmode_Attack)
			counter=0;
		asrMode=asrmode_Attack;	// we need to attack
		max_burst=tmp;
		length=lenAttack;
		if(length&&length!=counter)
		{
			// calculate the needed increment to fade during length
			amp_mul_increment=(valSustainLevel / max_burst - amp_mul)/(length-counter);
		}
	  }

	  // prev_data is used to get the rate of change for the signal
  	  Prev_data=*auxsample;

	  //
	  // envelope, attack part is in the burst detection
	  //

mode_loop:
	  if(asrMode)
	  {
	    if(++counter>length)
		{
			counter=0;
			// some mode has ended, we need to find to which mode we're going to change
			switch(asrMode)
			{
			  case asrmode_Attack:
				  // here we set the values for sustain
				asrMode=asrmode_Sustain;
				length=lenSustain;
				amp_mul=(double)valSustainLevel/max_burst;
				amp_mul_increment=0;
			  break;
			  case asrmode_Sustain:
				  // here's settings for release
				max_burst=0;
				asrMode=asrmode_Release;
				length=lenRelease;
				if(length)
				{
				  amp_mul_increment=(1-amp_mul)/length;
				}
			  break;
			  case asrmode_Release:
				  // here's normal action
				amp_mul_increment=0;
				amp_mul=1;
				asrMode=asrmode_Dry;
			  break;
			}
			goto mode_loop; // this checks if the length of some parameter is 0
		}

	    amp_mul+=amp_mul_increment;
	  }

	  //
	  // apply envelope
	  //

	  (*sample)*=(float)amp_mul;
}

inline void CTrack::miniWorkTrack(float *sample)
{
	  float tmp=(float)fabs(*sample);
	  // if tmp goes ever the threshold or the rise is bigger then valDelta then we need to trigger attack
	  if(tmp>valThreshold || fabs(*sample - Prev_data) > valDelta)
	  {
		counter=0;
		length=lenAttack;
	    asrMode=asrmode_Attack;	// we need to attack
	  }
	  if(asrMode)
	  {
	    if(counter++>=length)
		{
			asrMode=asrmode_Dry;
		} else
		{
			*sample=0;
		}
	  }
}

/*
inline void mi::analysein(float psample)
{
	dcnum++;
	In_dcsum+=psample;
	if(psample>In_maxlevelfound)
	{
		In_maxlevelfound=psample;
	}
	if(psample<In_minlevelfound)
	{
		In_minlevelfound=psample;
	}
}

inline void mi::analyseout(float psample)
{
	Out_dcsum+=psample;
	if(psample>Out_maxlevelfound)
	{
		Out_maxlevelfound=psample;
	}
	if(psample<Out_minlevelfound)
	{
		Out_minlevelfound=psample;
	}
}

*/

bool mi::Work(float *psamples, int numsamples, int const mode)
{
	int c,d;
	c=numsamples;

	if (mode == WM_WRITE || mode == WM_NOIO)
		return false;

	if(mode == WM_READ)	// this makes the asr counters move even while <thru>:ing
	{
		for(d=0;d < numTracks;d++)
		{
			Tracks[d].counter+=numsamples;
		}
	} else
	{

	switch(running_mode)
	{
	case 0:
	//
	// analyse, no inline code
	//
		while(c--)
		{
			dcnum++;
			In_dcsum+=*psamples;
			if(*psamples>In_maxlevelfound)
			{
				In_maxlevelfound=*psamples;
			}
			if(*psamples<In_minlevelfound)
			{
				In_minlevelfound=*psamples;
			}
			psamples++;
		}
		break;
	//
	// ana + prot
	//
	case 1:
		while(c--)
		{
			dcnum++;
			In_dcsum+=*psamples;
			if(*psamples>In_maxlevelfound)
			{
				In_maxlevelfound=*psamples;
			}
			if(*psamples<In_minlevelfound)
			{
				In_minlevelfound=*psamples;
			}
			Tracks[0].miniWorkTrack(psamples);
			Out_dcsum+=*psamples;
			if(*psamples>Out_maxlevelfound)
			{
				Out_maxlevelfound=*psamples;
			}
			if(*psamples<Out_minlevelfound)
			{
				Out_minlevelfound=*psamples;
			}
			psamples++;
		}
		break;
	//
	// analyse + asr
	//
	case 2:
auxbusfallback_ana:
	  while(c--)
	  {
			dcnum++;
			In_dcsum+=*psamples;
			if(*psamples>In_maxlevelfound)
			{
				In_maxlevelfound=*psamples;
			}
			if(*psamples<In_minlevelfound)
			{
				In_minlevelfound=*psamples;
			}
			for(d=0;d < numTracks;d++)
			{
				Tracks[d].WorkTrack(psamples);
			}
			Out_dcsum+=*psamples;
			if(*psamples>Out_maxlevelfound)
			{
				Out_maxlevelfound=*psamples;
			}
			if(*psamples<Out_minlevelfound)
			{
				Out_minlevelfound=*psamples;
			}
		psamples++;
	  }
	  break;
	//
	// ana + auxbus asr
	//
	case 3:
	    if (Channel != -1)
		{
		  float *tmpbuf = new float [numsamples];
		  AB_Receive(Channel, tmpbuf, numsamples);

	      while(c--)
		  {
			dcnum++;
			In_dcsum+=*psamples;
			if(*psamples>In_maxlevelfound)
			{
				In_maxlevelfound=*psamples;
			}
			if(*psamples<In_minlevelfound)
			{
				In_minlevelfound=*psamples;
			}
			Tracks[0].auxWorkTrack(psamples,tmpbuf);
		    for(d=1;d < numTracks;d++)
			{
			  Tracks[d].WorkTrack(psamples);
			}
			Out_dcsum+=*psamples;
			if(*psamples>Out_maxlevelfound)
			{
				Out_maxlevelfound=*psamples;
			}
			if(*psamples<Out_minlevelfound)
			{
				Out_minlevelfound=*psamples;
			}
		    psamples++;
			tmpbuf++;
		  }
		} else
		{
			goto auxbusfallback_ana;
		}
	break;
	//
	// auxbus asr
	//
	case 4:
	    if (Channel != -1)
		{
		  float *tmpbuf = new float [numsamples];
		  AB_Receive(Channel, tmpbuf, numsamples);

	      while(c--)
		  {
			 Tracks[0].auxWorkTrack(psamples,tmpbuf);
		     for(d=1;d < numTracks;d++)
			 {
			   Tracks[d].WorkTrack(psamples);
			 }
		     psamples++;
			 tmpbuf++;
		  }
		} else
		{
			goto auxbusfallback;
		}
	break;
	//
	// asr
	//
	case 5:
auxbusfallback:
	  while(c--)
	  {
		for(d=0;d < numTracks;d++)
		{
			Tracks[d].WorkTrack(psamples);
		}
		psamples++;
	  }
	  break;
	//
	// ana + prot
	//
	case 6:
		while(c--)
		{
			Tracks[0].miniWorkTrack(psamples);
			Out_dcsum+=*psamples;
			psamples++;
		}
		break;
	}
    }
  return true;
}
